﻿import React, { Fragment, useState, useEffect, useRef } from "react";
import ReactDOM from "react-dom";
import { Viewer } from "@mescius/activereportsjs-react";
import "@mescius/activereportsjs/pdfexport";
import "@mescius/activereportsjs/htmlexport";
import "@mescius/activereportsjs/tabulardataexport";
import "@mescius/activereportsjs-i18n";
import { PageReport } from "@mescius/activereportsjs/core";

function App() {
  const [params, setParams] = useState([]);
  const [showParamBar, setShowParamBar] = useState(false);
  const [paramValues, setParamValues] = useState({});
  const viewerRef = useRef(null);
  const pageReportRef = useRef(null);

  const createParamControls = () => {
    return params.map((param) => {
      const paramType = param.dataType.toLowerCase();
      return (
        <div key={param.name} className="form-group">
          <label>{param.label}</label>
          {paramType === "boolean" ? (
            <input
              type="checkbox"
              checked={paramValues[param.name] || false}
              onChange={(e) => handleParamChange(param.name, e.target.checked)}
            />
          ) : paramType === "date" ? (
            <input
              type="date"
              className="form-control"
              value={paramValues[param.name] || ""}
              onChange={(e) => handleParamChange(param.name, e.target.value)}
            />
          ) : paramType === "datetime" || paramType === "datetime-local" ? (
            <input
              type="datetime-local"
              className="form-control"
              value={paramValues[param.name] || ""}
              onChange={(e) => handleParamChange(param.name, e.target.value)}
            />
          ) : typeof param.value === "number" ? (
            <input
              type="number"
              className="form-control"
              value={paramValues[param.name] || ""}
              onChange={(e) => handleParamChange(param.name, e.target.value)}
            />
          ) : (
            <input
              type="text"
              className="form-control"
              value={paramValues[param.name] || ""}
              onChange={(e) => handleParamChange(param.name, e.target.value)}
            />
          )}
        </div>
      );
    });
  };

  const handleParamChange = (name, value) => {
    setParamValues((prev) => ({
      ...prev,
      [name]: value,
    }));
  };

  const openReport = async (rptName) => {
    const report = new PageReport();
    pageReportRef.current = report;

    await report.load(rptName);

    const parameterDescriptors = report.reportParameters.getDescriptors();

    const fetchedParams = Object.keys(parameterDescriptors).map((paramName) => {
      const descriptor = parameterDescriptors[paramName];
      return {
        name: paramName,
        label: descriptor.prompt || paramName,
        value: descriptor.defaultValue,
        dataType: descriptor.dataType,
      };
    });

    setParams(fetchedParams);
    setShowParamBar(true);
  };

  const onPreview = async () => {
    try {
      const applySteps = params.map((param) => ({
        name: param.name,
        value: paramValues[param.name],
      }));

      await pageReportRef.current.reportParameters.applySteps(applySteps);

      const reportParams = params.map((param) => ({
        Name: param.name,
        Value: Array.isArray(paramValues[param.name])
          ? paramValues[param.name]
          : [paramValues[param.name]],
      }));

      viewerRef.current.Viewer.open("reports/TestReport.rdlx-json", {
        ReportParams: reportParams,
      });
    } catch (error) {
      console.error("Error applying parameters:", error);
    }
  };

  useEffect(() => {
    openReport("reports/TestReport.rdlx-json");
  }, []);

  return (
    <Fragment>
      <div className="container-fluid">
        <button
          className="btn btn-outline-secondary mt-3"
          onClick={() => setShowParamBar(!showParamBar)}
        >
          {showParamBar ? "Hide Parameters" : "Show Parameters"}
        </button>

        {showParamBar && (
          <div className="form-group mb-3 mt-3">
            <h4>Custom Parameters</h4>
            {createParamControls()}
            <div className="mt-1">
              <button
                type="button"
                className="btn btn-outline-primary"
                onClick={onPreview}
              >
                Preview Report
              </button>
            </div>
          </div>
        )}
      </div>

      <div id="viewer-host">
        <Viewer
          theme="ActiveReports"
          ref={viewerRef}
          documentLoaded={() => setShowParamBar(false)}
          reportLoaded={() => setShowParamBar(true)}
        />
      </div>
    </Fragment>
  );
}

ReactDOM.render(<App />, document.getElementById("root"));
